// SetIPAddress.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include "ntddndis.h"
#include "winioctl.h"
#include <iptypes.h>
#include "iphlpapi.h"

#define	LAN1_REGPATH	L"Comm\\PCI\\RTCENIC1\\Parms\\TcpIp"
#define	LAN2_REGPATH	L"Comm\\PCI\\RTCENIC2\\Parms\\TcpIp"

void SetNewIPAddress(DWORD nLanNo, WCHAR *szIP, WCHAR *szNetMask)
{
	HKEY hKey;
	DWORD dwDHCP;
	ULONG ulRet;

	if ( nLanNo == 1 )
	{
		// Open the Registry for the new IP assignment
		ulRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE, LAN1_REGPATH, 0, 0, &hKey);
	} else {
		// Open the Registry for the new IP assignment
		ulRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE, LAN2_REGPATH, 0, 0, &hKey);
	}
	if ( ulRet == ERROR_SUCCESS )
	{
		// disable DHCP
		dwDHCP = 0;
		ulRet = RegSetValueEx( hKey, TEXT("EnableDHCP"), NULL, REG_DWORD, (const BYTE*)(&dwDHCP), sizeof(DWORD) );

		// assign IP address string into the specific Registry
		ulRet = RegSetValueEx( hKey, TEXT("IpAddress"), NULL, REG_SZ, (const BYTE*)szIP, wcslen(szIP)*2 );

		// assign Sub Net mask string into the specific Registry
		ulRet = RegSetValueEx( hKey, TEXT("Subnetmask"), NULL, REG_SZ, (const BYTE*)szNetMask, wcslen(szNetMask)*2 );

		//ulRet = RegSetValueEx( hKey, TEXT("DefaultGateway"), NULL, REG_SZ, (const BYTE*)szGateway, wcslen(szGateway)*2 );
		//ulRet = RegSetValueEx( hKey, TEXT("DNS"), NULL, REG_SZ, (const BYTE*)szDNS, wcslen(szDNS)*2 );
		//ulRet = RegSetValueEx( hKey, TEXT("WINS"), NULL, REG_SZ, (const BYTE*)szWINS, wcslen(szWINS)*2 );

		// Let the registry change become effective immediately 
		RegFlushKey( hKey );

		// close registry handle
		RegCloseKey( hKey );
	}
}

BOOL DoNdisIOControl(DWORD dwCommand, LPVOID pInBuffer, DWORD cbInBuffer, LPVOID pOutBuffer, DWORD * pcbOutBuffer)
{
   HANDLE hNdis;
   BOOL   fResult = FALSE;

   hNdis = CreateFile(DD_NDIS_DEVICE_NAME, GENERIC_READ | GENERIC_WRITE,
                        FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_ALWAYS,
                        0, NULL);

   if (INVALID_HANDLE_VALUE != hNdis) 
   {
      fResult = DeviceIoControl(hNdis, dwCommand, pInBuffer, cbInBuffer,
                                  pOutBuffer, (pcbOutBuffer ? *pcbOutBuffer : 0),
                                  pcbOutBuffer, NULL);
      CloseHandle(hNdis);
   }
   return fResult;
}

void RestartNetwork(void)
{
    WCHAR      wszAdapterNames[512];
    WCHAR*     pInterface = NULL;
    
    // Get the array of adapters
    DWORD cbAdapterNames = sizeof(wszAdapterNames);
    if ( !DoNdisIOControl(IOCTL_NDIS_GET_ADAPTER_NAMES, NULL, 0, wszAdapterNames, &cbAdapterNames) ) 
	{
        return;
    }

    pInterface = wszAdapterNames;
    while(_wcsicmp(pInterface, L"") != 0)
    {
        // Check if the adapter is valid
        // If the adapter is not enabled and it is bound that means it is
        // not a valid LAN connectoid
        TCHAR szAdapterBindings[512] = {0};
        DWORD cbAdapterBindings = sizeof(szAdapterBindings);
        if(!DoNdisIOControl(IOCTL_NDIS_GET_ADAPTER_BINDINGS,
                            pInterface, 
                            (wcslen(pInterface)+1)*sizeof(WCHAR),
                            szAdapterBindings, 
                            &cbAdapterBindings)) {
	        return;
        }
        
        BOOL fEnabled = FALSE;
        
        // Loop through all of the bindings for this adapter looking for tcp/ip
        TCHAR* pCurrentBinding = szAdapterBindings;
        while(_T('\0') != *pCurrentBinding)
        {
           if(!::_tcsicmp(pCurrentBinding, _T("TCPIP"))     ||
              !::_tcsicmp(pCurrentBinding, _T("80211SIMP")) ||
              !::_tcsicmp(pCurrentBinding, _T("MBRIDGE"))   ||
              !::_tcsicmp(pCurrentBinding, _T("MSTCP"))) 
           {
              fEnabled = TRUE;
              break;
           }
           pCurrentBinding += ::_tcslen(pCurrentBinding)+1;
        }

        // If the binding is disabled and exists then we will not add it.
        if(fEnabled || !(*szAdapterBindings))
        {
            WCHAR wmszTmp[100];

            // Rebind IOCTL takes a multiSz
			memset(wmszTmp,0,sizeof(wmszTmp));
            wcscpy(wmszTmp, pInterface);
            wmszTmp[wcslen(wmszTmp)+1] = 0x00;
            
            DoNdisIOControl(IOCTL_NDIS_REBIND_ADAPTER, 
                            wmszTmp,
                           (wcslen(wmszTmp)+2) * sizeof(WCHAR),
                           NULL, 
                           NULL);

            // Give NDIS some time before next rebind
            Sleep(1000);
        }

        pInterface += wcslen(pInterface) + 1;
    }
}

int _tmain(int argc, _TCHAR* argv[])
{
	// set LAN1 IP
	SetNewIPAddress(1, L"172.16.100.1", L"255.255.255.0");

	// set LAN2 IP
	SetNewIPAddress(2, L"172.16.200.1", L"255.255.255.0");

	// Restart network, so that the new setting can be take effect.
	RestartNetwork();

	return 0;
}

